#!/bin/sh
# version 2.0 for BVA v2.0
# autor MAYDOKIN Aleksey < aleksey [at] maydokin [dot] ru >

if [ $# -lt 1 ]
then
  echo 'Project name does not specified!'
  exit
fi

NAME=${1//[^0-9a-z_]/}

DIR=/var/vhosts

PATHTONGINXCFG=/etc/nginx
PATHTOAPACHECFG=/etc/httpd/conf.d

NGINXCNF=$(cat <<_EOF
server {

	listen 80;
	server_name $NAME;
	server_name_in_redirect off;
	access_log $DIR/$NAME/log/nginx_access.log common;
	index index.php;
	error_page 500 502 503 504 /500.html;
	error_page 404 = /404.php;
	proxy_set_header X-Real-IP \$remote_addr;
	proxy_set_header X-Forwarded-For \$proxy_add_x_forwarded_for;
	proxy_set_header Host \$host:80;
	client_max_body_size 1024M;
	client_body_buffer_size 4M;
	root $DIR/$NAME/www;

	if (-f $DIR/$NAME/www/.htsecure) {
		rewrite ^(.*)\$ https://\$host\$1 permanent;
	}

	location / {
		expires 3d; 
		if (\$request_method = OPTIONS ) {
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = PROPFIND ) {
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = PROPPATCH ) {   
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = MKCOL ) {
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = COPY ) {
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = MOVE ) {
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = LOCK ) {
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = UNLOCK ) {
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$request_method = PUT ) {
			proxy_pass http://127.0.0.1:8888;
		}
	}

	location ~ ^(/extranet/docs|/docs|/workgroups|/company/profile|/bitrix/tools|/company/personal/user).*/\$ {
		proxy_pass http://127.0.0.1:8888;
	}

	location ~ ^(/extranet/docs|/docs|/workgroups|/company/profile|/bitrix/tools|/company/personal/user) {
		if (-d \$request_filename) {
			rewrite ^(.*)(/*)\$  \$1/  last;
		}
		proxy_pass http://127.0.0.1:8888;
	}

	location ~ ^(/bitrix/html_pages) {
		root $DIR/$NAME/www;
		index index@.html;
		if (!-f \$request_filename) {
			rewrite ^/bitrix/html_pages(.*)@(.*)\.html\$ \$1.php?\$2 break;
			rewrite ^/bitrix/html_pages(.*)\.html\$ \$1\.php break;
			proxy_pass http://127.0.0.1:8888;
		}
	}

	location ~ \.php\$ {
		root $DIR/$NAME/www;
		if (\$request_method = POST ) {
			break;
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$http_cookie !~ "PHPSESSID=" ) {
			rewrite ^(.*)\.php\$ /bitrix/html_pages\$1@\$args.html? last;
		}
		proxy_pass http://127.0.0.1:8888;
	}

	location ~ /\$ {
		root $DIR/$NAME/www;
		if (\$request_method = POST ) {
			break;
			proxy_pass http://127.0.0.1:8888;
		}
		if (\$http_cookie !~ "PHPSESSID=" ) {
			rewrite ^(.*)/\$ /bitrix/html_pages\$1/index@\$args.html? last;
		}
		proxy_pass http://127.0.0.1:8888;
	}

	location ~ (/|\.php|\.asmx)\$ {
		proxy_pass http://127.0.0.1:8888;
	}
	location ~ /\.ht {
		deny all;
	}
	location ~ /favicon.ico {
		proxy_pass http://127.0.0.1:8888;
	}
	
	location ~ ^(/bitrixsetup\.php)\$ {
		proxy_pass http://127.0.0.1:8888;
		proxy_buffering off;
	}

}
_EOF
)

APACHECNF=$(cat <<_EOF
<VirtualHost *:8888>

	ServerName $NAME

	ServerAdmin webmaster@localhost
	DocumentRoot $DIR/$NAME/www

	<Directory />
		Options FollowSymLinks
		AllowOverride None
	</Directory>

	<Directory $DIR/$NAME/www/>
		Options Indexes FollowSymLinks MultiViews
		AllowOverride All
		DirectoryIndex index.php index.html index.htm
		Order allow,deny
		allow from all
	</Directory>

	ErrorLog $DIR/$NAME/log/apache2_error.log
	CustomLog $DIR/$NAME/log/apache2_access.log combined
	LogLevel warn

	<IfModule mod_rewrite.c>
		RewriteEngine On
		RewriteCond %{HTTP:HTTPS} =YES
		RewriteRule .* - [E=HTTPS:on,L]
	</IfModule>

</VirtualHost>
_EOF
)

MYSQLUSR=root
MYSQLPWD=

echo 'Creating directories'
mkdir -p $DIR/$NAME/www $DIR/$NAME/log
if [ $? -ne 0 ]
then
  echo 'Cannot create the directories'
  echo "Error  code:$?"
  exit
fi

echo 'Changing rights on the directories'
chown -R bitrix:bitrix $DIR/$NAME

echo 'Creating simbolic link'
ln -s $DIR/$NAME /home/bitrix

echo 'Creating nginx config'
cat > $PATHTONGINXCFG/sites-available/$NAME << _EOF
$NGINXCNF
_EOF

if [ $? -ne 0 ]
then
  echo 'Cannot create nginx config'
  echo "Error  code:$?"
  exit
fi

ln -s $PATHTONGINXCFG/sites-available/$NAME $PATHTONGINXCFG/sites-enabled/$NAME

echo 'Restarting nginx'
service nginx restart

echo 'Creating apache config'
cat > $PATHTOAPACHECFG/sites-available/$NAME << _EOF
$APACHECNF
_EOF

if [ $? -ne 0 ]
then
  echo 'Cannot create apache config'
  echo "Error  code:$?"
  exit
fi

ln -s $PATHTOAPACHECFG/sites-available/$NAME $PATHTOAPACHECFG/sites-enabled/$NAME

echo 'Restarting Zend Server'
service zend-server restart

echo 'Creating database'
export MYSQL_PWD=$MYSQLPWD
mysql -u$MYSQLUSR -e "CREATE DATABASE \`$NAME\`"

if [ $? -ne 0 ]
then
  echo 'Cannot create database'
  echo "Error  code:$?"
  unset MYSQL_PWD
  exit
fi


mysql -u$MYSQLUSR -e "GRANT ALL ON \`$NAME\`.* TO \`$NAME\`@localhost IDENTIFIED BY '$NAME'"

if [ $? -ne 0 ]
then
  echo 'Cannot create database user'
  echo "Error  code:$?"
  unset MYSQL_PWD
  exit
fi

unset MYSQL_PWD

echo "Virtual host $NAME created successfully!"